{

  open Lexing
  open C_parser

  let keywords = 
    let l = 
      [ "module"          , MODULE ;
	"let"             , LET ;
	"begin"           , BEGIN ;
	"end"             , END ; 
	"if"              , IF ;
	"then"            , THEN;
	"else"            , ELSE;
	"true"            , TRUE ;
	"false"           , FALSE ;
	"while"           , WHILE ;
	"do"              , DO ;
	"done"            , DONE ;
	"repeat"          , REPEAT;
	"until"           , UNTIL ;
	"return"          , RETURN ;
	"mark"            , MARK ; 
	"unmark"          , UNMARK ;
	"pickup"          , PICKUP ;
	"move"            , MOVE ;
	"drop"            , DROP ;
	"turn"            , TURN ;
	"goto"            , GOTO ; 
	"left"            , LEFT ; 
	"right"           , RIGHT ;
	"sense"           , SENSE ;
	"here"            , HERE ;
	"ahead"           , AHEAD ; 
	"leftahead"           , LEFTAHEAD ; 
	"rightahead"           , RIGHTAHEAD ; 
	"flip"            , FLIP ;
	"orelse"          , ORELSE ;
	"friend"          , FRIEND ;
	"foe"             , FOE ;
	"friendwithfood"  , FRIENDWITHFOOD ;
	"foewithfood"     , FOEWITHFOOD ;
	"food"            , FOOD ;
	"rock"            , ROCK ;
	"foemarker"       , FOEMARKER ;
	"home"            , HOME ;
	"foehome"         , FOEHOME ]   
    in
    let t = Hashtbl.create 53 in
      List.iter (fun (kw,tok) -> Hashtbl.add t kw tok) l ;
      t

  exception Syntax_error of int
  let line=ref 1
}

let ident = ['a' - 'z' 'A' - 'Z' ] ['a' - 'z' 'A' - 'Z' '0' - '9' '_']*

rule token = parse
  | [' ' '\t'] +
      { token lexbuf }
  | '\n' { incr line; token lexbuf }
  | ';' { PV }
  | '=' { EQUAL } 
  | '(' { LP } 
  | ')' { RP } 
  | ':' { DP }
  | ',' { VIR }
  | "&&" { ET }
  | "||" { OU }
  | "not" { NON }
  | "#0" { MARKER(0) }
  | "#1" { MARKER(1) }
  | "#2" { MARKER(2) }
  | "#3" { MARKER(3) }
  | "#4" { MARKER(4) }
  | "#5" { MARKER(5) }
  | ['0'-'9']+ { INT(int_of_string (Lexing.lexeme lexbuf)) }
  | ident { let s = lexeme lexbuf in
	      try Hashtbl.find keywords s with Not_found -> ID(s) }
  | "(*" {comment 0 lexbuf}
  | eof { EOF }
  | _   { raise (Syntax_error !line) }
and comment n = parse
  | "*)" {if n<=0 then token lexbuf else comment (n-1) lexbuf}
  | "(*" {comment (n+1) lexbuf}
  | '\n' {incr line; comment n lexbuf} 
  | eof  {raise (Syntax_error (-1))}
  | _    {comment n lexbuf}
  
