%{

  open Ast

%}

%start prog
%type <Ast.prog> prog 

/*s Tokens */

/* Literals */

%token MODULE LET IF THEN ELSE WHILE DO DONE REPEAT UNTIL RETURN 
%token MARK UNMARK PICKUP MOVE DROP TURN GOTO
%token FRIEND FOE FRIENDWITHFOOD FOEWITHFOOD FOOD ROCK FOEMARKER HOME FOEHOME
%token ET OU NON TRUE FALSE ORELSE HERE AHEAD LEFTAHEAD RIGHTAHEAD 
%token LEFT RIGHT SENSE FLIP 
%token <int> MARKER
%token <int> INT
%token <string> ID
%token PV DP LP RP VIR BEGIN END EQUAL 
%token EOF EOL

%nonassoc THEN
%nonassoc ELSE

%left ET
%left OU
%right NON

%%

prog:
| main EOF { {main=$1;procs=[]} }
| procedure prog { let s=$2 in {s with procs=$1::s.procs} }
;

main:
| MODULE EQUAL body { $3 }
;

procedure:
| LET ID args EQUAL body { {name=$2;args=$3;body=$5} }
;

args:
| LP RP {[]}
| LP arg_list RP {$2}
;

arg_list:
| ID VIR arg_list {$1::$3}
| ID { [$1] } 
;

body:
| instr PV body { $1::$3 }
| ID DP body { (Lbl $1)::$3 }
| instr {[$1]} 
;

linstr:
| ID DP instr {[Lbl $1;$3]}
| instr {[$1]}
;

instr:
| IF bool_expr THEN linstr ELSE linstr { If($2,$4,$6) }
| IF bool_expr THEN linstr { If($2,$4,[]) }
| WHILE bool_expr DO body DONE { While($2,$4) }
| REPEAT body UNTIL bool_expr { Repeat($4,$2) }
| MARK MARKER  { Mark($2) }
| UNMARK MARKER { Unmark($2) }
| PICKUP ORELSE ID { Pickup($3) }
| MOVE ORELSE ID { Move($3) }
| DROP { Drop }
| TURN left_or_right { Turn($2) }
| GOTO ID { Goto($2) }
| RETURN {Return}
| ID args {Call($1,$2)}  
| LP body RP {Seq($2)}
| BEGIN body END {Seq($2)}
;

left_or_right:
| LEFT { Left }
| RIGHT { Right }
;

bool_expr:
| TRUE {True}
| FALSE {False}
| NON bool_expr { Not($2) }
| bool_expr ET bool_expr { And($1,$3) }
| bool_expr OU bool_expr { Or($1,$3) }
| pred { Pred($1) }
| LP bool_expr RP { $2 }
;

pred:
| FLIP INT { Flip($2) }
| condition sense_dir { Sense($1,$2) }
;

sense_dir:
| HERE { Here }
| AHEAD { Ahead }
| LEFTAHEAD { ALeft }
| RIGHTAHEAD { ARight }
;

condition:
| FRIEND         { Friend }
| FOE            { Foe }
| FRIENDWITHFOOD { FriendWithFood }
| FOEWITHFOOD    { FoeWithFood }
| FOOD           { Food }
| ROCK           { Rock }
| MARKER         { Marker($1) }
| FOEMARKER      { FoeMarker }
| HOME           { Home }
| FOEHOME        { FoeHome }
;

