
open Ast
open Boolean_simplifier

let gen_lbl=
  let a=ref 0 in
    fun () -> incr a;("@@"^(string_of_int !a)) 

type flat_instr=
    FLbl of label
  | FGoto of label
  | FCond of pred * label * label  (* pred includes flip *)
  | FMark of int
  | FUnmark of int
  | FPickup of label
  | FDrop
  | FTurn of left_or_right
  | FMove of label

let rec flatten_bool lbltrue lblfalse accu = function
    True -> FGoto lbltrue::accu
  | False -> FGoto lblfalse::accu
  | Pred p -> FCond (p,lbltrue,lblfalse)::accu
  | Not c ->  flatten_bool lblfalse lbltrue accu c
  | And(c1,c2) -> 
      let tmp = gen_lbl () in
      let accu1 = (FLbl tmp)::(flatten_bool lbltrue lblfalse accu c2) in
	flatten_bool tmp lblfalse accu1 c1
  | Or(c1,c2) -> 
      let tmp = gen_lbl () in
      let accu1 = (FLbl tmp)::(flatten_bool lbltrue lblfalse accu c2) in
	flatten_bool lbltrue tmp accu1 c1

let rec flatten_instr i accu = 
  match i with
      If (c,b1,b2) ->
	let endif = gen_lbl () in
	let lelse = gen_lbl () in
	let lthen = gen_lbl () in
	let accu1= FLbl lelse::
		    (flatten_body b2 (FLbl endif::accu)) in
	let accu2= FLbl lthen::
		    (flatten_body b1 (FGoto endif::accu1)) in
	  flatten_bool lthen lelse accu2 c
    | While(c,b)->
	let loop  = gen_lbl () in
	let ldo   = gen_lbl () in
	let ldone = gen_lbl () in
	let accu1 = (FGoto loop)::(FLbl ldone)::accu in
	let accu2 = (FLbl ldo)::(flatten_body b accu1) in
	  (FLbl loop)::(flatten_bool ldo ldone accu2 c)
    | Repeat(c,b)->
	let loop  = gen_lbl () in
	let ldone = gen_lbl () in
	let accu1 = (FLbl ldone)::accu in
	let accu2 = (flatten_bool ldone loop accu1 c) in
	  (FLbl loop)::(flatten_body b accu2)
    | Mark i -> FMark i :: accu
    | Unmark i -> FUnmark i :: accu
    | Pickup l -> FPickup l :: accu
    | Move l -> FMove l :: accu
    | Drop -> FDrop :: accu
    | Turn lr -> FTurn lr :: accu
    | Goto l -> FGoto l :: accu
    | Seq b -> flatten_body b accu
    | Lbl l -> FLbl l :: accu
    | Call (_,_) | Return -> assert false
and flatten_body b accu=
  List.fold_right flatten_instr b accu

let flatten_prog p = flatten_body p []
	    

