
open Format
open Ast
open Flatten
open Resolve

let string_cond = function
    Friend -> "Friend"
  | Foe -> "Foe"
  | FriendWithFood -> "FriendWithFood"
  | FoeWithFood -> "FoeWithFood"
  | Food -> "Food"
  | Rock -> "Rock"
  | Marker i -> "Marker"^(string_of_int i)
  | FoeMarker -> "FoeMarker"
  | Home -> "Home"
  | FoeHome -> "FoeHome"

let string_dir = function
    Ahead -> "Ahead"
  | Here -> "Here"
  | ARight -> "RightAhead"
  | ALeft -> "LeftAhead"

let string_lr = function
    Left -> "Left"
  | Right -> "Right"

let print_flat_instr fmt = function
    FLbl l -> fprintf fmt "%s :\n" l
  | FGoto l -> fprintf fmt "\tGoto %s\n" l 
  | FCond (Sense(cond,dir),l1,l2) -> 
      fprintf fmt "\tSense %s %s %s %s\n" 
      (string_dir dir) l1 l2 (string_cond cond);
  | FCond (Flip i,l1,l2) -> 
      fprintf fmt "\tFlip %d %s %s\n" i l1 l2
  | FMark i -> fprintf fmt "\tMark %d\n" i
  | FUnmark i -> fprintf fmt "\tUnmark %d\n" i
  | FPickup l -> fprintf fmt "\tPickUp %s\n" l
  | FDrop -> fprintf fmt "\tDrop\n"
  | FTurn lr -> fprintf fmt "\tTurn %s\n" (string_lr lr) 
  | FMove l -> fprintf fmt "\tMove %s\n" l

let print_flat_prog fmt prg = 
  List.iter (print_flat_instr fmt) prg;
  Format.printf "@?"

let print_ant_brain fmt prg=
  let (find_ad,find_next) = resolve_lbl prg in
  let state_cnt = ref 0 in
  let print_ant_state state_cnt cont= function
      FLbl l -> ()
    | FGoto l -> ()
    | FCond (Sense(cond,dir),l1,l2) ->
	let ad1=find_ad l1 in
	let ad2=find_ad l2 in
	  fprintf fmt "Sense %s %d %d %s \t; state %d\n" 
	    (string_dir dir) ad1 ad2 (string_cond cond) !state_cnt;
	  incr state_cnt
    | FCond (Flip i,l1,l2) -> 
	let ad1=find_ad l1 in
	let ad2=find_ad l2 in
	  fprintf fmt "Flip %d %d %d \t\t; state %d\n" i ad1 ad2 !state_cnt;
	  incr state_cnt
    | FMark i ->
	let ad=find_next (1+ !state_cnt) cont in
	  fprintf fmt "Mark %d %d \t\t; state %d\n" i ad !state_cnt;
	  incr state_cnt
    | FUnmark i ->	
	let ad=find_next (1+ !state_cnt) cont in
	  fprintf fmt "Unmark %d %d \t\t; state %d\n" i ad !state_cnt;
	  incr state_cnt
    | FPickup l -> 
	let ad=find_next (1+ !state_cnt) cont in
	let ad0=find_ad l in
	  fprintf fmt "PickUp %d %d \t\t; state %d\n" ad ad0 !state_cnt;
	  incr state_cnt  
    | FDrop ->      
	let ad=find_next (1+ !state_cnt) cont in
	  fprintf fmt "Drop %d \t\t\t; state %d\n" ad !state_cnt;
	  incr state_cnt
    | FTurn lr -> 
	let ad=find_next (1+ !state_cnt) cont in
	  fprintf fmt "Turn %s %d \t\t; state %d\n" 
	    (string_lr lr) ad !state_cnt;
	  incr state_cnt
    | FMove l ->
	let ad=find_next (1 + !state_cnt) cont in
	let ad0=find_ad l in
	  fprintf fmt "Move %d %d \t\t; state %d\n" ad ad0 !state_cnt;
	  incr state_cnt in
  let rec dump = function
      [] -> 
	fprintf fmt "Flip 1 %d %d \t\t; The Bottomless Pit !!\n" 
	!state_cnt !state_cnt 
    | fi::cont -> 
	print_ant_state state_cnt cont fi;
	dump cont in
    dump prg



