{

  open Lexing
  open Parser

  exception Error of string
  
  let keywords = 
    let l = 
      [ "mark" , MARK ;
	"unmark" ,  UNMARK ; 
	"pickup" , PICKUP ; 
	"drop" , DROP ;
	"lturn" , LTURN ; 
	"rturn" , RTURN ;
	"move" , MOVE ;
	"flip" , FLIP ;
	"here" , HERE ;
	"ahead" , AHEAD ;
	"lahead" , LEFTAHEAD ; 
	"rahead" , RIGHTAHEAD ;
	"friend" , FRIEND ;
	"foe" , FOE ;
	"friendwithfood" , FRIENDWITHFOOD ;
	"forwithfood" , FOEWITHFOOD ;
	"food" , FOOD ;
	"rock" , ROCK ;
	"marker" , MARKER ;
	"foemarker" , FOEMARKER ;
	"home" , HOME ;
	"foehome" , FOEHOME ]
    in 
    let t = Hashtbl.create 31 in
      List.iter (fun (kwd,tok) -> Hashtbl.add t kwd tok) l; t 

}

let espaces = [' ' '\t' '\n']+
let lettre = ['a'-'z' 'A'-'Z']
let comment = "//" [^ '\n']* '\n'
let chiffre = ['0'-'9']
let entier = chiffre+
let ident = lettre (lettre|chiffre|'_')*

rule token = parse
  | espaces        { token lexbuf }
  | entier         { INT (int_of_string (lexeme lexbuf)) }
  | ident          { let s = lexeme lexbuf in 
		       try Hashtbl.find keywords s 
		       with Not_found -> IDENT s }
  | comment        { token lexbuf }      
  | '+'            { PLUS }
  | '!'            { BOUCLE }
  | '?'            { NOPBOUCLE }
  | '~'            { NON }
  | '.'            { POINT }
  | '='            { EQ }
  | '['            { CG }
  | ']'            { CD }
  | '('            { LP }
  | ')'            { RP }
  | '*'            { STAR }
  | eof            { EOF }
  | _              { raise (Error (lexeme lexbuf)) }
