
type left_or_right = Left | Right

type sense_dir = 
  | Here           (* sense the ant's current cell *)
  | Ahead          (* sense the cell straight ahead in the direction ant is facing *)
  | LeftAhead      (* sense the cell that would be ahead if ant turned left *)
  | RightAhead     (* sense the cell that would be ahead if ant turned right *)

type marker = int (* 0..5 *)

type condition =
  | Friend             (* cell contains an ant of the same color *)
  | Foe                (* cell contains an ant of the other color *)
  | FriendWithFood     (* cell contains an ant of the same color carrying food *)
  | FoeWithFood        (* cell contains an ant of the other color carrying food *)
  | Food               (* cell contains food (not being carried by an ant) *)
  | Rock               (* cell is rocky *)
  | Marker of marker   (* cell is marked with a marker of this ant's color *)
  | FoeMarker          (* cell is marked with *some* marker of the other color *)
  | Home               (* cell belongs to this ant's anthill *)
  | FoeHome            (* cell belongs to the other anthill *)

type state = int (* 0 .. 9999 *)

type instruction =
  | Sense of sense_dir * state * state * condition  
  | Mark of marker * state
  | Unmark of marker * state
  | PickUp of state * state
  | Drop of state
  | Turn of left_or_right * state
  | Move of state * state
  | Flip of int * state * state

open Format

let pr_sd fmt = function
  | Ahead -> fprintf fmt "Ahead"
  | LeftAhead -> fprintf fmt "LeftAhead"
  | RightAhead -> fprintf fmt "RightAhead"
  | Here -> fprintf fmt "Here"
;;

let pr_c fmt = function
  | Friend -> fprintf fmt "Friend" 
  | Foe -> fprintf fmt "Foe"              
  | FriendWithFood -> fprintf fmt "FriendWithFood"   
  | FoeWithFood -> fprintf fmt "FoeWithFood"      
  | Food -> fprintf fmt "Food"             
  | Rock -> fprintf fmt "Rock"             
  | Marker n -> fprintf fmt "Marker %d" n 
  | FoeMarker -> fprintf fmt "FoeMarker"        
  | Home -> fprintf fmt "Home"             
  | FoeHome -> fprintf fmt "FoeHome"          
;;

let pr_d fmt = function
  | Left -> fprintf fmt "Left"
  | Right -> fprintf fmt "Right"
;;

let output ch l =
  let fmt = formatter_of_out_channel ch in
  List.iter
    (fun i ->
       match i with
	 | Sense(sd,s1,s2,c) -> 
	     fprintf fmt "Sense %a %d %d %a@\n" pr_sd sd s1 s2 pr_c c
	 | Mark(m,s) ->
	     fprintf fmt "Mark %d %d@\n" m s
	 | Unmark(m,s) -> 
	     fprintf fmt "Unmark %d %d@\n" m s
	 | PickUp(s1,s2) -> 
	     fprintf fmt "PickUp %d %d@\n" s1 s2
	 | Drop(s) -> 
	     fprintf fmt "Drop %d@\n" s
	 | Turn(d,s) -> 
	     fprintf fmt "Turn %a %d@\n" pr_d d s
	 | Move(s1,s2) -> 
	     fprintf fmt "Move %d %d@\n" s1 s2
	 | Flip(n,s1,s2) -> 
	     fprintf fmt "Flip %d %d %d@\n" n s1 s2)
    l


