
type left_or_right = Left | Right

type sense_dir = 
  | Here           (* sense the ant's current cell *)
  | Ahead          (* sense the cell straight ahead in the direction ant is facing *)
  | LeftAhead      (* sense the cell that would be ahead if ant turned left *)
  | RightAhead     (* sense the cell that would be ahead if ant turned right *)

type marker = int (* 0..5 *)

type condition =
  | Friend             (* cell contains an ant of the same color *)
  | Foe                (* cell contains an ant of the other color *)
  | FriendWithFood     (* cell contains an ant of the same color carrying food *)
  | FoeWithFood        (* cell contains an ant of the other color carrying food *)
  | Food               (* cell contains food (not being carried by an ant) *)
  | Rock               (* cell is rocky *)
  | Marker of marker   (* cell is marked with a marker of this ant's color *)
  | FoeMarker          (* cell is marked with *some* marker of the other color *)
  | Home               (* cell belongs to this ant's anthill *)
  | FoeHome            (* cell belongs to the other anthill *)

type state = int (* 0 .. 9999 *)

type instruction =
  | Sense of sense_dir * state * state * condition  
  | Mark of marker * state
  | Unmark of marker * state
  | PickUp of state * state
  | Drop of state
  | Turn of left_or_right * state
  | Move of state * state
  | Flip of int * state * state

val output : out_channel -> instruction list -> unit;;
