
open Printf

type ant = { 
  name : string;
  file : string;
  mutable score : int
}

let debug = ref false
let nb_world = ref 10
let ants = ref []

let spec =
  ["-debug", Arg.Set debug, "debug mode";
   "-nbworld", Arg.Int ((:=) nb_world), "<number of world>"]
let usage = "usage: tournoi [options] directory"

let add_ant f = 
  if not (Filename.check_suffix f ".ant") then begin 
    Arg.usage spec usage; exit 1 
  end;
  let a = 
    { name = Filename.chop_suffix (Filename.basename f) ".ant";
      file = f;
      score = 0; }
  in
  ants := a :: !ants

let () = Arg.parse spec add_ant usage

let ants = Array.of_list !ants

let write_scores () =
  printf "----------------\n";
  Array.sort (fun a1 a2 -> a2.score - a1.score) ants;
  Array.iter (fun a -> printf "%s: %d\n" a.name a.score) ants

let main () =
  for w = 0 to !nb_world - 1 do
    let world = sprintf "test/sample%d.world" w in
    printf "world = %s\n" world; flush stdout;
    Array.iteri 
      (fun i red ->
	 Array.iteri
	 (fun j black ->
	    if i <> j then begin
	      printf "%s vs %s: " red.name black.name; flush stdout;
	      let cmd = sprintf "./sim -world %s -red %s -black %s > out"
			       world red.file black.file in
	      if !debug then begin eprintf "cmd = %s\n" cmd; flush stderr end;
	      let e = Sys.command cmd in
	      if e <> 0 then 
		printf "sim failed (exited with code %d)\n" e
	      else begin
		try
		  let c = open_in "out" in
		  let r = input_line c in
		  close_in c;
		  printf "%s\n" r; flush stdout;
		  match r with
		    | "RED" -> 
			red.score <- red.score + 2
		    | "BLACK" -> 
			black.score <- black.score + 2
		    | "DRAW" -> 
			red.score <- red.score + 1; 
			black.score <- black.score + 1
		    | _ -> 
			raise Exit
		  with _ ->
			printf "bad output from sim (ignored)\n"; flush stdout
	      end
	    end)
	 ants)
      ants
  done

let () = 
  try
    main ();
    write_scores ()
  with e ->
    eprintf "anomaly: %s\n" (Printexc.to_string e); flush stderr;
    write_scores ()


(***
	      let args = 
		[| "-world"; world; "-red"; red.file; "-black"; black.file |] 
	      in
	      let c,new_out = Unix.pipe () in
	      let new_in = Unix.descr_of_in_channel (open_in "/dev/null") in
	      let new_err = Unix.descr_of_out_channel (open_out "/dev/null") in
	      let pid = Unix.create_process cmd args new_in new_out new_err in
	      let _ = Unix.waitpid [] pid in
	      let r = input_line (Unix.in_channel_of_descr c) in
***)
